@@ -20,6 +20,7 @@ use anyhow::{anyhow, Context, Result};
 use app::SortColumn;
 use app::{App, Mode};
 use bpf_program::BpfProgram;
+use clap::Parser;
 use crossterm::event::{self, poll, Event, KeyCode, KeyModifiers};
 use crossterm::execute;
 use crossterm::terminal::{
@@ -35,11 +36,13 @@ use ratatui::style::{Color, Modifier, Style, Stylize};
 use ratatui::text::Line;
 use ratatui::widgets::{
     Axis, Block, BorderType, Borders, Cell, Chart, Dataset, GraphType, Padding, Paragraph, Row,
-    Table,
+    Scrollbar, ScrollbarOrientation, Table,
 };
 use ratatui::{symbols, Frame, Terminal};
 use std::fs;
 use std::io::{self, Stdout};
+use std::mem::MaybeUninit;
+use std::ops::{Add, Mul};
 use std::os::fd::{FromRawFd, OwnedFd};
 use std::panic;
 use std::time::Duration;
@@ -68,9 +71,28 @@ const SORT_INFO_FOOTER: &str = "(Esc) back";
 
 const PROCFS_BPF_STATS_ENABLED: &str = "/proc/sys/kernel/bpf_stats_enabled";
 
+const TABLE_HEADER_HEIGHT: u16 = 1;
+const TABLE_HEADER_MARGIN: u16 = 1;
+const TABLE_ROW_HEIGHT: u16 = 1;
+const TABLE_ROW_MARGIN: u16 = 1;
+const TABLE_FOOTER_HEIGHT: u16 = 1; // derived from `TABLE_FOOTER`
+
+#[derive(Parser, Debug)]
+#[command(
+    name = env!("CARGO_PKG_NAME"),
+    version = env!("CARGO_PKG_VERSION"),
+    long_version = concat!(
+        env!("CARGO_PKG_VERSION"), " created by ",
+        "Jose Fernandez"
+    ),
+    author = "Jose Fernandez",
+    about = env!("CARGO_PKG_DESCRIPTION"),
+    override_usage = "sudo bpftop"
+)]
+struct Bpftop {}
+
 impl From<&BpfProgram> for Row<'_> {
     fn from(bpf_program: &BpfProgram) -> Self {
-        let height = 1;
         let cells = vec![
             Cell::from(bpf_program.id.to_string()),
             Cell::from(bpf_program.bpf_type.to_string()),
@@ -81,7 +103,7 @@ impl From<&BpfProgram> for Row<'_> {
             Cell::from(format_percent(bpf_program.cpu_time_percent())),
         ];
 
-        Row::new(cells).height(height as u16).bottom_margin(1)
+        Row::new(cells).height(TABLE_ROW_HEIGHT).bottom_margin(TABLE_ROW_MARGIN)
     }
 }
 
@@ -114,13 +136,21 @@ impl Drop for TerminalManager {
 }
 
 fn main() -> Result<()> {
+    let _ = Bpftop::parse();
+
     if !nix::unistd::Uid::current().is_root() {
         return Err(anyhow!("This program must be run as root"));
     }
 
+    // Initialize the journald layer or ignore if not available
+    let journald_layer = match tracing_journald::layer() {
+        Ok(layer) => Some(layer),
+        Err(_) => None,
+    };
+
     // Initialize the tracing subscriber with the journald layer
     let registry = tracing_subscriber::registry()
-        .with(tracing_journald::layer()?)
+        .with(journald_layer)
         .with(tracing_subscriber::filter::LevelFilter::INFO);
     // Try to set this subscriber as the global default
     registry.try_init()?;
@@ -144,7 +174,8 @@ fn main() -> Result<()> {
 
         // load and attach pid_iter BPF program to get process information
         let skel_builder = PidIterSkelBuilder::default();
-        let open_skel = skel_builder.open()?;
+        let mut open_object = MaybeUninit::uninit();
+        let open_skel = skel_builder.open(&mut open_object)?;
         let mut skel = open_skel.load()?;
         skel.attach()?;
         iter_link = skel.links.bpftop_iter;
@@ -265,7 +296,7 @@ fn run_draw_loop<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> Result
 }
 
 fn ui(f: &mut Frame, app: &mut App) {
-    let rects = Layout::vertical([Constraint::Min(5), Constraint::Length(3)]).split(f.size());
+    let rects = Layout::vertical([Constraint::Min(5), Constraint::Length(3)]).split(f.area());
 
     match app.mode {
         Mode::Table | Mode::Filter | Mode::Sort => render_table(f, app, rects[0]),
@@ -351,8 +382,8 @@ fn render_graphs(f: &mut Frame, app: &mut App, area: Rect) {
         .bounds([0.0, cpu_y_max])
         .labels(vec![
             "0%".into(),
-            ((cpu_y_max / 2.0).to_string() + "%").into(),
-            (cpu_y_max.to_string() + "%").into(),
+            ((cpu_y_max / 2.0).to_string() + "%"),
+            (cpu_y_max.to_string() + "%"),
         ]);
     let cpu_chart = Chart::new(cpu_datasets)
         .block(
@@ -382,8 +413,8 @@ fn render_graphs(f: &mut Frame, app: &mut App, area: Rect) {
         .bounds([0.0, eps_y_max])
         .labels(vec![
             "0".into(),
-            ((eps_y_max / 2.0).to_string()).into(),
-            (eps_y_max.to_string()).into(),
+            ((eps_y_max / 2.0).to_string()),
+            (eps_y_max.to_string()),
         ]);
     let eps_chart = Chart::new(eps_datasets)
         .block(
@@ -413,8 +444,8 @@ fn render_graphs(f: &mut Frame, app: &mut App, area: Rect) {
         .bounds([0.0, runtime_y_max])
         .labels(vec![
             "0".into(),
-            ((runtime_y_max / 2.0).to_string()).into(),
-            (runtime_y_max.to_string()).into(),
+            ((runtime_y_max / 2.0).to_string()),
+            (runtime_y_max.to_string()),
         ]);
     let runtime_chart = Chart::new(runtime_datasets)
         .block(
@@ -525,6 +556,18 @@ fn render_table(f: &mut Frame, app: &mut App, area: Rect) {
 
     let rows: Vec<Row> = items.iter().map(|item| item.into()).collect();
 
+    let content_height: u16 = TABLE_HEADER_HEIGHT
+        .add(TABLE_HEADER_MARGIN)
+        .add((rows.len() as u16).mul(TABLE_ROW_HEIGHT.add(TABLE_ROW_MARGIN)))
+        .add(TABLE_FOOTER_HEIGHT);
+    if content_height > area.height {
+        // content exceeds screen size; display scrollbar
+        app.vertical_scroll_state = app.vertical_scroll_state.content_length(rows.len());
+    } else {
+        // content fits on screen; hide scrollbar
+        app.vertical_scroll_state = app.vertical_scroll_state.content_length(0);
+    }
+
     let widths = [
         Constraint::Percentage(5),
         Constraint::Percentage(17),
@@ -542,9 +585,14 @@ fn render_table(f: &mut Frame, app: &mut App, area: Rect) {
                 .borders(Borders::ALL)
                 .title(" eBPF programs "),
         )
-        .highlight_style(selected_style)
+        .row_highlight_style(selected_style)
         .highlight_symbol(">> ");
     f.render_stateful_widget(t, area, &mut app.table_state);
+    f.render_stateful_widget(
+        Scrollbar::new(ScrollbarOrientation::VerticalRight).thumb_symbol("░"),
+        area,
+        &mut app.vertical_scroll_state,
+    );
 }
 
 fn render_footer(f: &mut Frame, app: &mut App, area: Rect) {
@@ -590,10 +638,10 @@ fn render_footer(f: &mut Frame, app: &mut App, area: Rect) {
             f.render_widget(filter_footer, split_area[0]);
 
             // Displays cursor when inputting
-            f.set_cursor(
+            f.set_cursor_position((
                 split_area[0].x + filter_input.visual_cursor() as u16 + 2,
                 split_area[0].y + 1,
-            );
+            ));
             drop(filter_input);
         }
         Mode::Sort => {
